using OpenTK.Mathematics;
using OpenTK.Windowing.Common.Input;

namespace GraphicsManager.Objects.Core;

public class SubHitBox
{
    public virtual Vector2i Size { get; set; }
    public virtual Vector2i Location { get; set; }
    public virtual MouseCursor HoverMouse { get; set; } = MouseCursor.Default;
    private bool mi = false;

    internal virtual bool MouseInside
    {
        get
        {
            return mi;
        }
        set
        {
            if (mi == value) return;
            mi = value;
            if (value && MouseEnter is not null) MouseEnter.Invoke(this).Wait();
            if (!value && MouseLeave is not null) MouseLeave.Invoke(this).Wait();
        }
    }
    
    public event Func<SubHitBox, Task>? Clicked;
    public event Func<SubHitBox, Task>? MouseLeave;
    public event Func<SubHitBox, Task>? MouseEnter;

    internal void SendClick()
    {
        if (Clicked is not null) _ = Clicked.Invoke(this);
    }

    public override string ToString()
    {
        return base.ToString()+ $": {{X: {Location.X}, Y: {Location.Y}, Width: {Size.X}, Height: {Size.Y}}}";
    }
}